package com.macro.mall.tiny.modules.business.service.impl;

import com.macro.mall.tiny.common.util.MyFileUtils;
import com.macro.mall.tiny.config.UploadConfig;
import com.macro.mall.tiny.domain.AdminUserDetails;
import com.macro.mall.tiny.modules.business.model.BDirectory;
import com.macro.mall.tiny.modules.business.model.BFile;
import com.macro.mall.tiny.modules.business.model.BFileDetail;
import com.macro.mall.tiny.modules.business.service.BDirectoryCacheService;
import com.macro.mall.tiny.modules.business.service.FileService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.EnableAsync;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.util.List;

import static com.macro.mall.tiny.common.util.UploadUtils.*;


/**
 * 文件上传服务
 */
@Service
@Slf4j
@EnableAsync
public class FileServiceImpl implements FileService {

    @Autowired
    private AsyncHandler asyncHandler;

    @Autowired
    BDirectoryCacheService directoryCacheService;

    /**
     * 上传文件
     *
     * @param file
     */
    public void upload(MultipartFile file) throws IOException {
        String path = UploadConfig.path + file.getOriginalFilename();
        MyFileUtils.write(path, file.getInputStream());
    }

    /**
     * 分块上传文件
     *
     * @param size
     * @param chunks
     * @param chunk
     * @param file
     * @throws IOException
     */
    public void uploadWithBlock(String name,
                                String lineName,
                                Long provinceId,
                                Long size,
                                Integer chunks,
                                Integer chunk,
                                MultipartFile file,
                                AdminUserDetails userDetails) throws IOException {
        putChunkMap(name, chunks);
        String target = UploadConfig.tempPath + name;
        MyFileUtils.writeWithBlock(target, size, file.getInputStream(), file.getSize(), chunks, chunk);
        addChunk(name, chunk);
        if (isUploaded(name)) {
            removeKey(name);
            asyncHandler.handleDocument(provinceId, target, lineName, userDetails);
        }
    }

    @Override
    public BDirectory getRootDirectory(AdminUserDetails userDetails) {
        BDirectory directory = directoryCacheService.getDirectory(userDetails);
        if (directory != null) {
            return directory;
        }
        directory = MyFileUtils.getRootDirectory(userDetails);
        if (directory != null) {
            directoryCacheService.setDirectory(userDetails, directory);
        } else {
            directoryCacheService.setDirectory(userDetails, new BDirectory());
        }
        return directory;
    }

    @Override
    public List<BFileDetail> getFileDetailList(String path, AdminUserDetails userDetails) {
        return MyFileUtils.getFileDetailList(path, userDetails);
    }

    @Override
    public List<BFile> getFileList(String path, AdminUserDetails userDetails) {
        return MyFileUtils.getSonFiles(path, userDetails);
    }

}
